==> jungle-1.0.tar.gz: jungle-1.0/jungle/apps/timereport/badges.py <==
import ftplib
from datetime import datetime, time
from jungle.timereport.models import Transit, Reason, TimetableProfile, BadgeReader
from jungle.registration.models import UserProfile
from jungle.functions.utils import DictWithDefault
from django.conf import settings
from django.db import transaction
from jungle.templatetags.jungle_tags import ugettext as _

class MissingProfile(Exception): pass

class NotSaved(Exception): pass

DEFAULT_REASON = Reason.objects.get(code='0000')
transits_added = None
transits_missing_profile = None
transits_errors = None


def setup():
    global Profiles
    global Users
    global R
    Profiles, Users = get_profiles()
    R = get_reasons()

def get_data_from_ftp(location=None):
    setup()
    #r = BadgeReader.get(badge_reader)
    readers = BadgeReader.objects.filter(status=1)
    if location:
        readers = readers.filter(location=location)
        
    for reader in readers:
        ftp = ftplib.FTP(reader.ip, reader.user, reader.password)
        ftp.set_pasv(reader.passive)
        ftp.retrlines('RETR ' + 'ltcom.trn', handle_transit)

def get_data_from_file(filename):
    setup()
    f = open(filename)
    for line in reversed(f.readlines()):
        handle_transit(line)
            
def handle_transit(line):
    line = line.strip('\n')
    info = parse(line)
    try:
        add_transit(line, info)
    except Exception, e:
        if isinstance(e, MissingProfile):
            tell_missing_profile(line, info)
        else:
            tell_errors(line, info, e)

def tell_missing_profile(line, info):
    global transits_missing_profile
    if not transits_missing_profile:
        transits_missing_profile = open(settings.CWD + '/data/missing_profile', 'a')
    transits_missing_profile.write("%(line)s %(date_time)s %(badge_num)s\n" % info)
        
def tell_errors(line, info, e):
    global transits_errors
    if not transits_errors:
        transits_errors = open(settings.CWD + '/data/handle_error', 'a')
    transits_errors.write("%s\n%s\n" % (line,vars(e)))

def get_profiles():
    Users = DictWithDefault(default=None)
    ## FIXME: defaul='impiegato' non ha senso se non per le prove
    #default_profile = TimetableProfile.objects.get(name='impiegato')
    #Profiles = DictWithDefault()
    Profiles = {}

    for u in UserProfile.objects.all():
        Users[u.badge_num] = u.user
        Profiles[u.badge_num] = u.time_profile
    return (Profiles, Users)

def get_reasons():
    R = DictWithDefault(default=DEFAULT_REASON)
    for r in Reason.objects.all():
        R[r.code] = r
    return R

def parse(line):
    i={'line' : line}
    if line[0] == 'E':
        i['direction'] = 1
    elif line[0] == 'U':
        i['direction'] = 2
    i['type'] = line[1]
    i['g_sett'] = line[2]
    i['badge_num'] = int(line[3:12])
    i['reason'] = line[12:16]
    i['hh'] = int(line[16:18])
    i['min'] = int(line[18:20])
    i['sec'] = int(line[20:22])
    i['dd'] = int(line[22:24])
    i['mm'] = int(line[24:26])
    i['year'] = 2000 + int(line[26:28] )
    i['badge_reader'] = int(line[28:30])
    i['date_time'] = datetime(i['year'], i['mm'], i['dd'], i['hh'], i['min'])
    return i

def add_transit(line, i):
    #i = parse(line)
    if i['badge_num'] not in Profiles:
        raise MissingProfile("Badge n. %s does not have a related profile" % i['badge_num'])
    
    day_reference, odv = guess_odv_from_transit(
        i['date_time'], Profiles[i['badge_num']],
        i['direction'], Users[i['badge_num']])
    
    if odv:
        tr_type = i['type']
    else:
        tr_type = 10  # extra not specified if entrance or exit

    tr = Transit(
        status=1,
        reason=R[i['reason']],
        date_reference = day_reference,
        user=Users[i['badge_num']],
        date_time=i['date_time'],
        direction=i['direction'],
        type=tr_type,
        badge_num=i['badge_num'],
        badge_reader_id = i['badge_reader'],
        profile = Profiles[i['badge_num']],
        odv=odv        )
    tr.save()
    if not tr.id:
        raise NotSaved('Not saved, probably integrity error')
    ### log the data
    global transits_added
    if not transits_added:
        transits_added = open(settings.CWD + '/data/transits_added', 'a')
    transits_added.write(line+"\n")
    
    return tr
        
def guess_odv_from_transit(date_time, profile, direction, user):
    """lookup the profile and profile_detail to guess which odv is this transit
    Return a tuple (day, odv)
    When improved to cope with over midnight turns day may be one day before
    date_time.date() if is the end of a turn tat began the previous day.
    """
    ### FIXME: not good for night turns!!!
    TRUST_DIRECTION = getattr(settings, 'TIMEREPORT_TRUST_DIRECTION', True)
        
    isoweekday = date_time.isoweekday()
    details = profile.timetableprofiledetail_set.filter(isoweekday=isoweekday)
    if TRUST_DIRECTION:
        details = details.filter(direction=direction)

    odv = {}
    day = date_time.date()
    for d in details:
        ## check with all official times of the date and get the one that
        ## is closer (min delta from official)

        ## first consider entrance (time)
        official_datetime  = datetime.combine(day, d.time)
        delta = date_time - official_datetime
        odv[abs(delta)] = d.odv

        ## now consider exit (time + duration)
        official_datetime  = datetime.combine(day, d.time_out)
        delta = date_time - official_datetime
        odv[abs(delta)] = d.odv + 1

    if odv:
        min_delta = min(odv.keys())
        #print 'MIN', min_delta, odv[min_delta], "\n"
        return (day, odv[min_delta])
    else:
        #  STRAORDINARIO
        # so it's out of official timetable
        # let's consider it inbound if no other inbound are present
        # print "EXTRA", user, date_time.strftime("%d/%m: %H:%M")
        ins = Transit.cron_objects.filter(
            date_reference=date_time.date(), user=user)

        odv = len(ins) + 1
#         if not ins:  # entrance: morning or afternoon?
#             if date_time.time() < time(12):
#                 odv = 1
#             else:
#                 odv = 3
#         else: # exit
#             if date_time.time() < time(14):
#                 odv = 2
#             else:
#                 odv = 4
        #print "no timedetail for profile %s, %s day N. %s" % (
        #    profile, date_time, isoweekday)
        return (day, odv)

def send_email_if_needed():
    global transits_missing_profile 
    global transits_errors

    if not settings.ADMINS and settings.ADMINS[1]:
        ## the following should get cought by cron log
        print _("Coudn't tell admins of mistakes becouse ADMINS is not configured")
        return
    
    if transits_missing_profile :
        from django.core.mail import send_mail
        message = _("While importing data from badge readers a profile was addressed that culd not be resolved into a user")
        send_mail(_("Missing Profile"), message + "\n%s" % transits_missing_profile, '', [settings.ADMINS[0][1]])


    if transits_errors :
        from django.core.mail import send_mail
        message = _("Errors while reading data from badge reader")
        send_mail(_("Errors in Badge Reader"), message + "\n%s" % transit_errors, '', [settings.ADMINS[0][1]])

==> jungle-1.0.1.tar.gz: jungle-1.0.1/jungle/apps/timereport/badges.py <==
import logging
import logging.handlers
import ftplib
from datetime import datetime, time
from jungle.timereport.models import Transit, Reason, TimetableProfile, BadgeReader
from jungle.registration.models import UserProfile
from jungle.functions.utils import DictWithDefault
from django.conf import settings
from django.db import transaction
from jungle.templatetags.jungle_tags import ugettext as _

class MissingProfile(Exception):
    pass

class NotSaved(Exception):
    pass

DEFAULT_REASON = Reason.objects.get(code='0000')
transits_added = None
transits_missing_profile = None
transits_errors = None

def configure_logging(name, console=False):
    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)

    formatter = logging.Formatter('%(name)s - %(levelname)s - %(message)s')
    handler = logging.handlers.SysLogHandler(address='/dev/log')
    handler.setFormatter(formatter)
    logger.addHandler(handler)

    if console:
        ch = logging.StreamHandler()
        ch.setLevel(logging.DEBUG)
        logger.addHandler(ch)

    return logger

logger = configure_logging('badge reader')
#logger.info("Inizio")

def setup():
    global Profiles
    global Users
    global R
    Profiles, Users = get_profiles()
    R = get_reasons()

def get_data_from_ftp(location=None):
    setup()
    #r = BadgeReader.get(badge_reader)
    readers = BadgeReader.objects.filter(status=1)
    if location:
        readers = readers.filter(location=location)

    for reader in readers:
        ftp = ftplib.FTP(reader.ip, reader.user, reader.password)
        ftp.set_pasv(reader.passive)
        ftp.retrlines('RETR ' + 'ltcom.trn', handle_transit)

def get_data_from_file(filename):
    setup()
    f = open(filename)
    for line in reversed(f.readlines()):
        handle_transit(line)

def handle_transit(line):
    logger.info("Line: %s" % line)
    line = line.strip('\n')
    info = parse(line)
    try:
        add_transit(line, info)
    except Exception as e:
        if isinstance(e, MissingProfile):
            tell_missing_profile(line, info)
        else:
            tell_errors(line, info, e)

def tell_missing_profile(line, info):
    global transits_missing_profile
    if not transits_missing_profile:
        transits_missing_profile = open(settings.CWD + '/data/missing_profile', 'a')
    transits_missing_profile.write("%(line)s %(date_time)s %(badge_num)s\n" % info)

def tell_errors(line, info, e):
    global transits_errors
    if not transits_errors:
        transits_errors = open(settings.CWD + '/data/handle_error', 'a')
    transits_errors.write("%s\n%s\n" % (line, vars(e)))

def get_profiles():
    Users = DictWithDefault(default=None)
    ## FIXME: defaul='impiegato' non ha senso se non per le prove
    #default_profile = TimetableProfile.objects.get(name='impiegato')
    #Profiles = DictWithDefault()
    Profiles = {}

    for u in UserProfile.objects.all():
        Users[u.badge_num] = u.user
        Profiles[u.badge_num] = u.time_profile
    return (Profiles, Users)

def get_reasons():
    R = DictWithDefault(default=DEFAULT_REASON)
    for r in Reason.objects.all():
        R[r.code] = r
    return R

def parse(line):
    i = {'line': line}
    if line[0] == 'E':
        i['direction'] = 1
    elif line[0] == 'U':
        i['direction'] = 2
    i['type'] = line[1]
    i['g_sett'] = line[2]
    i['badge_num'] = int(line[3:12])
    i['reason'] = line[12:16]
    i['hh'] = int(line[16:18])
    i['min'] = int(line[18:20])
    i['sec'] = int(line[20:22])
    i['dd'] = int(line[22:24])
    i['mm'] = int(line[24:26])
    i['year'] = 2000 + int(line[26:28])
    i['badge_reader'] = int(line[28:30])
    i['date_time'] = datetime(i['year'], i['mm'], i['dd'], i['hh'], i['min'])
    return i

def add_transit(line, i):
    #i = parse(line)
    if i['badge_num'] not in Profiles:
        raise MissingProfile("Badge n. %s does not have a related profile" % i['badge_num'])

    day_reference, odv = guess_odv_from_transit(
        i['date_time'], Profiles[i['badge_num']],
        i['direction'], Users[i['badge_num']])

    if odv:
        tr_type = i['type']
    else:
        tr_type = 10  # extra not specified if entrance or exit

    tr = Transit(
        status=1,
        reason=R[i['reason']],
        date_reference=day_reference,
        user=Users[i['badge_num']],
        date_time=i['date_time'],
        direction=i['direction'],
        type=tr_type,
        badge_num=i['badge_num'],
        badge_reader_id=i['badge_reader'],
        profile=Profiles[i['badge_num']],
        odv=odv)
    tr.save()
    logger.debug("Transit: %s" % tr)
    if not tr.id:
        raise NotSaved('Not saved, probably integrity error')
    ### log the data
    global transits_added
    if not transits_added:
        transits_added = open(settings.CWD + '/data/transits_added', 'a')
    transits_added.write(line+"\n")

    return tr

def guess_odv_from_transit(date_time, profile, direction, user):
    """lookup the profile and profile_detail to guess which odv is this transit
    Return a tuple (day, odv)
    When improved to cope with over midnight turns day may be one day before
    date_time.date() if is the end of a turn tat began the previous day.
    """
    ### FIXME: not good for night turns!!!
    TRUST_DIRECTION = getattr(settings, 'TIMEREPORT_TRUST_DIRECTION', True)

    isoweekday = date_time.isoweekday()
    details = profile.timetableprofiledetail_set.filter(isoweekday=isoweekday)
    if TRUST_DIRECTION:
        details = details.filter(direction=direction)

    odv = {}
    day = date_time.date()
    for d in details:
        ## check with all official times of the date and get the one that
        ## is closer (min delta from official)

        ## first consider entrance (time)
        official_datetime = datetime.combine(day, d.time)
        delta = date_time - official_datetime
        odv[abs(delta)] = d.odv

        ## now consider exit (time + duration)
        official_datetime = datetime.combine(day, d.time_out)
        delta = date_time - official_datetime
        odv[abs(delta)] = d.odv + 1

    if odv:
        min_delta = min(odv.keys())
        #print 'MIN', min_delta, odv[min_delta], "\n"
        return (day, odv[min_delta])
    else:
        #  STRAORDINARIO
        # so it's out of official timetable
        # let's consider it inbound if no other inbound are present
        # print "EXTRA", user, date_time.strftime("%d/%m: %H:%M")
        ins = Transit.cron_objects.filter(
            date_reference=date_time.date(), user=user)

        odv = len(ins) + 1
#         if not ins:  # entrance: morning or afternoon?
#             if date_time.time() < time(12):
#                 odv = 1
#             else:
#                 odv = 3
#         else: # exit
#             if date_time.time() < time(14):
#                 odv = 2
#             else:
#                 odv = 4
        #print "no timedetail for profile %s, %s day N. %s" % (
        #    profile, date_time, isoweekday)
        return (day, odv)

def send_email_if_needed():
    global transits_missing_profile
    global transits_errors

    if not settings.ADMINS and settings.ADMINS[1]:
        ## the following should get cought by cron log
        print _("Coudn't tell admins of mistakes becouse ADMINS is not configured")
        return

    if transits_missing_profile:
        from django.core.mail import send_mail
        message = _(
            "While importing data from badge readers a profile was addressed that culd not be resolved into a user")
        send_mail(_("Missing Profile"), message + "\n%s" %
                  transits_missing_profile.read(), '', settings.BADGE_ADMINS)

    if transits_errors:
        from django.core.mail import send_mail
        message = _("Errors while reading data from badge reader")
        send_mail(_("Errors in Badge Reader"), message + "\n%s" %
                  transits_errors, '', settings.BADGE_ADMINS)
